import numpy as np
from MatlabBlenderIO import csvtext2matrix
import csv


def query_csv(filepath, inquiry):
    # Querys .csv file for specific data.

    with open(filepath, mode='r') as file:
        data = file.read()
    
    data = data.replace("\n", "")
    data = data.replace("\n", "")
    data = data.split(",,,,,")
    data = list(map(lambda element: element.split(",,,,"), data))
    data = [list(filter(lambda item: item != '', element )) for element in data if type(element) is list]
    data = data[:-1]

    inquiry = data[0][0] + "." + inquiry
    full_inquiry = list(filter(lambda element: inquiry in element, data))
    index = data.index(full_inquiry[0])
    try:
        return csvtext2matrix(data[index][1])
    except:
        return data[index][1], 0





import mathutils as mt
import numpy as np
import MatlabBlenderIO as mlb
import os, bpy, time


def obj2csv(filepath, obj, *args, **kwargs):
    # Stores a BLender-object as a .csv-file.
    address  = kwargs.get('address',  None)
    if address == None:
        address = obj.name


    with open(filepath, 'a') as file:
        bpy.ops.object.select_all(action='DESELECT')
        obj.rotation_mode = 'QUATERNION'
        position = np.array(obj.location)
        attitude = obj.rotation_quaternion.to_matrix()
        attitude.transpose()
        attitude = np.array(attitude)
        file.write(address+",,,,,\n")
        file.write(address+".position,,,,\n")
        file.write(mlb.matrix2csvtext(position)+",,,,,\n")
        file.write(address+".attitude,,,,\n")
        file.write(mlb.matrix2csvtext(attitude)+",,,,,\n")

        if obj.type == 'MESH':
            file.write(address +'.mesh,,,,\n')
            file.write(obj.name+'.stl,,,,,\n')
            
            
            stl_filename       = obj.name+".stl"
            csv_filename       = filepath   .split('\\')[-1]
            stl_filepath       = filepath.replace(csv_filename, stl_filename)
            
            has_parent = not obj.parent == None
            
            # Un-parenting and saving logic
            if has_parent:
                parent     = obj.parent
                obj.parent = None

            obj.rotation_mode            = 'QUATERNION'
            original_rotation_quaternion = obj.rotation_quaternion.copy()
            original_location            = obj.location.copy()
            obj.rotation_quaternion      = mt.Quaternion((1,0,0,0))
            obj.location                 = mt.Vector    ((0,0,0))
            bpy.ops.object.select_all(action='DESELECT')
            obj.select_set(True)
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
            bpy.ops.export_mesh.stl(filepath=stl_filepath, use_selection=True, global_scale = 1)
            obj.select_set(False)
            obj.rotation_mode       = 'QUATERNION'
            obj.rotation_quaternion = original_rotation_quaternion
            obj.location            = original_location

            if has_parent:
                obj.parent              = parent

        
    for child in obj.children:
        mlb.obj2csv(filepath, child, address = address + '.' + child.name)
            


import numpy as np
import re
from itertools import groupby

def csvtext2matrix(text):
    text = str(text)
    text = text.replace("\n", "")
    if not "," in text:
        mat = np.array(float(text))
        dimension_lengths = 0
    else:

        # Find highest and lowest delimiters:
        highest_delimiter_number, lowest_delimiter_number = find_comma_sequences(text)
        highest_delimiter = highest_delimiter_number*","
        lowest_delimiter  = lowest_delimiter_number *","

        
        # Find the length of each dimension:
        dimensions         = highest_delimiter_number - lowest_delimiter_number+1
        dimension_lengths  = [0]*dimensions
        dimension_iterator = dimensions
        delimiter_iterator = highest_delimiter
        module             = 1

        while True:
            data_split_along_dim                     = text.split(delimiter_iterator)
            data_split_along_dim                     = list(filter( lambda x: x!="", data_split_along_dim ))
            dimension_lengths[dimension_iterator-1]  = int(len(data_split_along_dim)/module)
            module                                  *= dimension_lengths[dimension_iterator-1]

            if delimiter_iterator == lowest_delimiter:
                break
            else:
                delimiter_iterator  = delimiter_iterator[:-1]
                dimension_iterator -= 1

        # Making the matrix:
        flat_data = text.split(",")
        flat_data = list(filter( lambda x: x!="", flat_data ))
        flat_data = list(map(float, flat_data ))

        
        mat = np.array(flat_data)
        if dimensions > 1:
            mat = np.reshape(mat, dimension_lengths, 'F')

    return mat, dimension_lengths



def find_comma_sequences(text):
    comma_sequences = re.findall(r',+', text)
    if comma_sequences:
        longest  = max(len(seq) for seq in comma_sequences)
        smallest = min(len(seq) for seq in comma_sequences)
        return longest, smallest
    else:
        return 0, 0





def csvtext2matrix_demo():

    print(csvtext2matrix("0,0,0,,0,0,0,,,0,0,0,,0,0,0,,,0,0,0,,0,0,0"))
    print(csvtext2matrix("0,0,0,,0,0,0,,,0,0,0,,0,0,0"))
    print(csvtext2matrix("0,0,0,,0,0,0"))
    print(csvtext2matrix("0,1,3"))
    print(csvtext2matrix("0"))
    print(csvtext2matrix("0,1,0,,0,0,0,,,0,5,0,,8,0,0"))
    print(csvtext2matrix("0,,,1,,,2,,,4"))
    print(csvtext2matrix("0,,,,1,,,,2,,,,4"))
    testmat, testdims = csvtext2matrix("0,1,0,,0,0,0,,,0,5,0,,8,0,0")
    print(testmat[1][0][0])
    print(testmat[1][0][1])
    print(testmat[0][1][1])
    print(csvtext2matrix("0,1,0,0,0,0,,,0,5,0,8,0,0"))



csvtext2matrix_demo()


import os
from setuptools import setup, find_packages


EXCLUDED_DIRS = {".git", "__pycache__"}

def generate_init_files(package_dir):
    """
    Recursively generates __init__.py files in all package directories,
    automatically importing functions/classes from submodules and subdirectories.
    Excludes non-package directories like .git, __pycache__, and .egg-info.
    """
    for root, dirs, files in os.walk(package_dir):
        dirs[:] = [d for d in dirs if not d.endswith(".egg-info") and d not in EXCLUDED_DIRS]
        init_path = os.path.join(root, "__init__.py")

        if "__init__.py" not in files:
            with open(init_path, "w") as f:
                pass

        module_files = [f[:-3] for f in files if f.endswith(".py") and f != "__init__.py"]

        if module_files:
            with open(init_path, "w") as f:
                for module in module_files:
                    f.write(f"from .{module} import *\n")

        subdirectory_files = [d for d in dirs if os.path.isdir(os.path.join(root, d))]
        for subdir in subdirectory_files:
            subdir_init_path = os.path.join(root, subdir, "__init__.py")
            if not os.path.exists(subdir_init_path):
                with open(subdir_init_path, "w") as f:
                    pass

            with open(init_path, "a") as f:
                f.write(f"from .{subdir} import *\n")


def remove_init_files(directory):
    for root, dirs, files in os.walk(directory):
        init_path = os.path.join(root, '__init__.py')
        if os.path.exists(init_path):
            os.remove(init_path)


remove_init_files('MatlabBlenderIO')
generate_init_files('MatlabBlenderIO')




setup(
    name="MatlabBlenderIO",
    version="0.0.5",  # Increment your version
    packages=find_packages(),  # Start looking in MatRocket/MatRocket/
    include_package_data=True,  # Important for non-Python files
    install_requires=[],
    author="Vilgot Lötberg",
    author_email="vilgotl@kth.se",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/spiggen/MatlabBlenderIO",
    description="A solution for transferring data in and out of Blender and MATLAB, for example for ODE-solvers etc.",
    python_requires=">=3.6",
)


import numpy as np
import mathutils as mt
import MatlabBlenderIO as mlb
import csv, bpy


def csv2obj(filepath, *args, **kwargs):
    # Takes in the filepath to an object stored as a .csv file, and imports it into Blender.
    replace = kwargs.get('replace', None)
    if replace == None: replace = True

    with open(filepath, mode='r') as file:
        data = file.read()
    
    data = data.replace("\n", "")
    data = data.replace("\n", "")
    data = data.split(",,,,,")
    data = list(map(lambda element: element.split(",,,,"), data))
    data = [list(filter(lambda item: item != '', element )) for element in data if type(element) is list]
    data = data[:-1]
    
    
    # Loop through and create all the bodies:
    for element in data:
        is_body = len(element)==1
        if not is_body:
            is_mesh = "mesh" == element[0].split(".")[-1]
        else:
            is_mesh = False

        if is_body:
            body_name      = element[0].split(".")[-1]

            if body_name not in bpy.data.objects.keys():
                bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(0, 0, 0), scale=(1, 1, 1))
                body           = bpy.data.objects['Empty']
                body.name      = body_name

        if is_mesh:
            mesh_name   = element[0].split(".")[-2]
            if mesh_name not in bpy.data.objects.keys():
                new_mesh = True
            elif bpy.data.objects[mesh_name].type == 'EMPTY':
                bpy.ops.object.select_all(action='DESELECT')
                bpy.data.objects[mesh_name].select_set(True)
                bpy.ops.object.delete()
                bpy.ops.object.select_all(action='DESELECT')
                new_mesh = True
            else:
                new_mesh = False

            if new_mesh:
                stl_filename        = element[1].split('\\')[-1]
                csv_filename        = filepath  .split('\\')[-1]
                stl_filepath        = filepath  .replace(csv_filename, stl_filename)
                import_name         = stl_filename.replace(".stl", "")
                update_original     = False

                if import_name in bpy.data.objects.keys():
                    update_original = True
                    original        = bpy.data.objects[import_name]
                    original.name   = "Temp__"

                bpy.ops.import_mesh.stl(filepath = stl_filepath)
                bpy.data.objects[import_name].name = mesh_name 
                bpy.ops.object.select_all(action='DESELECT')

                if update_original:
                    original.name = import_name
    
    
    # Loop through and assign all the parent-relationships:
    for element in data:
        is_body = len(element)==1
        
        if is_body:
            address    = element[0].split(".")
            has_parent = len(address)>1
            body_name  = address[-1]
            
            if has_parent:
                parent_name = address[-2]
                bpy.data.objects[body_name].parent = bpy.data.objects[parent_name]

    
    # Loop through and assign position, rotation and mesh:
    for element in data:
        has_properties = len(element)>1
        if has_properties:
            body_name     = element[0].split(".")[-2]
            property_name = element[0].split(".")[-1]
            body          = bpy.data.objects[body_name]

            if property_name == "position":
 
                positions, position_dimensions = mlb.csvtext2matrix(element[1])

                if len(position_dimensions) > 2:
                    number_of_keyframes = positions.shape[2]
                    for keyframe in range(1,number_of_keyframes):
                        position = mt.Vector(positions[keyframe])
                        body.keyframe_insert('location', frame = keyframe)
                        body.location = position
                else:
                    position = mt.Vector(positions)
                    body.location = position
            
            if property_name == "attitude":
                body.rotation_mode = 'QUATERNION'
                attitudes, attitude_dimensions = mlb.csvtext2matrix(element[1])

                if len(attitude_dimensions) > 2:
                    number_of_keyframes = attitudes.shape[2]
                    for keyframe in range(1,number_of_keyframes):
                        attitude = attitudes[keyframe].invert()
                        attitude_quaternion = mt.Matrix(attitude).to_quaternion()
                        body.keyframe_insert('rotation_quaternion', frame = keyframe)
                        body.rotation_quaternion = attitude_quaternion
                else:
                    attitude_quaternion = mt.Matrix(attitudes).to_quaternion()
                    body.rotation_quaternion = attitude_quaternion
                    


import numpy as np
import math

def matrix2csvtext(mat,*args, **kwargs):
    # Converts a matrix to text that can be written to a .csv file, encoding dimension-information based on the number of ",".
    mat = np.array(mat)
    starting_delimiter    = kwargs.get('starting_delimiter', None)
    if starting_delimiter == None: starting_delimiter = ","

    text               = ""
    dimensions         = mat.ndim
    dimension_lenghts  = list(mat.shape)
    number_of_elements = mat.size
    flatmat            = mat.reshape(number_of_elements, 1)

    denominator_multiplier = [1]+dimension_lenghts

    for index in range(1,number_of_elements+1):
        text += str(float(flatmat[index-1][0]))

        if index != number_of_elements:
            text += starting_delimiter
            for dim in range(2,dimensions+1):
                if ((index)%(dimension_lenghts[dim-1]*math.prod(denominator_multiplier[:dim-1])) ) == 0: text += ","
            for dim in range(2,dimensions+1):
                if ((index)%(dimension_lenghts[dim-1]*math.prod(denominator_multiplier[:dim-1])) ) == 0: text += "\n"

    return text



def matrix2csvtext_demo():
    print(matrix2csvtext(np.array(  ((1,2,3),(4,5,6),(7,8,9))  )))
    print(matrix2csvtext(np.array(  (((1,2,3)),((4,5,6)),((7,8,9)))  )))
    print(matrix2csvtext(np.array(  (1,2,3)  )))
    print(matrix2csvtext(np.array(  (1)  )))
    print(matrix2csvtext(np.array(  (((1,2),(3,4)),((5,6),(7,8)))  )))
    print(matrix2csvtext(np.array(  ((1,2,3),(4,5,6),(7,8,9))  ), starting_delimiter = ",,"))
    print(matrix2csvtext(np.array(  (((1,2,3)),((4,5,6)),((7,8,9)))  ), starting_delimiter = ",,,"))
    print(matrix2csvtext(np.array(  (1,2,3)  ), starting_delimiter = ",,,,"))
    print(matrix2csvtext(np.array(  (1)  ), starting_delimiter = ",,,,,"))
    print(matrix2csvtext(np.array(  (((1,2),(3,4)),((5,6),(7,8)))  ), ","))

#matrix2csvtext_demo()